/*
 * THIS EXAMPLE CONTENT AND SOURCE IS PROVIDED ON AN "AS IS" BASIS. ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL GENUITEC LLC BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS EXAMPLE.
 */
package com.genuitec.pulse.server.delivery.extension;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUser;
import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUserAuthenticateRequest;
import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUserAuthenticateResponse;
import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUserLookupRequest;
import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUserLookupResponse;
import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUserRequest;
import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUserSearchRequest;
import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUserSearchResponse;
import com.genuitec.pulse.server.delivery.extension.user.DeliveryExternalUsersExtension;
import com.genuitec.pulse.server.delivery.extension.user.ExternalUserCapability;

public class DeliveryExternalUsersImpl extends DeliveryExternalUsersExtension {

	private static final String USER_ID_PREFIX = "user";

	@Override
	public Set<ExternalUserCapability> getExternalUserCapabilities() {
		Set<ExternalUserCapability> capabilities = new HashSet<ExternalUserCapability>();
		capabilities.add(ExternalUserCapability.USER_LOOKUP);
		return capabilities;
	}

	@Override
	public DeliveryExternalUserAuthenticateResponse validateUser(DeliveryExternalUserAuthenticateRequest request) {

		// Sanity check the user id & password
		String userId = request.getUserID();
		char[] password = request.getUserPassword();
		if (userId == null)
			return request.newInvalidUserResponse("User id must not be null and must be 4 characters long or greater");
		if (password == null)
			return request.newInvalidPasswordResponse("Password must be 4 characters long or greater");

		// Treat user ids as case insensitive
		String conditionedUserId = userId.trim().toLowerCase();

		// See if this looks like a valid user
		if (!conditionedUserId.toLowerCase().startsWith(USER_ID_PREFIX))
			return request.newInvalidUserResponse("There is no registered user matching [" + userId + "]");
		String intSpec = conditionedUserId.substring(USER_ID_PREFIX.length());
		int userNumber;
		try {
			userNumber = Integer.parseInt(intSpec);
		} catch (NumberFormatException e) {
			return request.newInvalidUserResponse("Invalid user number [" + intSpec + "]");
		}

		// Simulate validation of the password
		char[] actualPassword = ("password" + userNumber).toCharArray();
		if (!Arrays.equals(actualPassword, password))
			return request.newInvalidPasswordResponse("Invalid password for " + userId);

		// Success!
		return request.newSuccessResponse(simulateUser(request, userNumber));
	}

	@Override
	public DeliveryExternalUserSearchResponse searchUsers(DeliveryExternalUserSearchRequest request) {

		// Sanity check the user id & password
		String searchQuery = request.getSearchQuery();
		if (searchQuery.length() == 0 || searchQuery.equals("*") || searchQuery.equals(USER_ID_PREFIX + "*")) {
			return simulateUsers(request);
		}

		// Treat user ids as case insensitive
		String conditionedUserId = searchQuery.trim().toLowerCase();

		// See if this looks like a valid user
		if (!conditionedUserId.toLowerCase().startsWith(USER_ID_PREFIX))
			return request.newNoMatchesResponse("No results found, try '*' or '" + USER_ID_PREFIX + "*'");
		String intSpec = conditionedUserId.substring(USER_ID_PREFIX.length());
		int userNumber;
		try {
			userNumber = Integer.parseInt(intSpec);
		} catch (NumberFormatException e) {
			return request.newNoMatchesResponse("No results found, try '*' or '" + USER_ID_PREFIX + "*'");
		}

		SortedSet<DeliveryExternalUser> users = new TreeSet<DeliveryExternalUser>();
		users.add(simulateUser(request, userNumber));
		return request.newFoundResponse(users);
	}

	private DeliveryExternalUserSearchResponse simulateUsers(DeliveryExternalUserSearchRequest request) {
		SortedSet<DeliveryExternalUser> users = new TreeSet<DeliveryExternalUser>();
		for (int i = 0; i < 50; i++) {
			users.add(simulateUser(request, i));
		}
		return request.newFoundResponse(users);
	}

	@Override
	public DeliveryExternalUserLookupResponse lookupUsers(DeliveryExternalUserLookupRequest request) {
		SortedSet<DeliveryExternalUser> users = new TreeSet<DeliveryExternalUser>();
		for (String next : request.getUserIDs()) {
			String conditionedUserId = next.toLowerCase();
			if (!conditionedUserId.toLowerCase().startsWith(USER_ID_PREFIX))
				continue;
			String intSpec = conditionedUserId.substring(USER_ID_PREFIX.length());
			int userNumber;
			try {
				userNumber = Integer.parseInt(intSpec);
			} catch (NumberFormatException e) {
				continue;
			}
			users.add(simulateUser(request, userNumber));
		}
		return request.newSuccessResponse(users);
	}

	private DeliveryExternalUser simulateUser(DeliveryExternalUserRequest request, int userNumber) {
		DeliveryExternalUser user = request.newUser(USER_ID_PREFIX + userNumber);
		user.setDisplayName("Long Name " + userNumber);
		return user;
	}
}